#!/usr/bin/python

import os, sys
import re
import httplib, urllib

cookiefile = os.path.expanduser('~%s/.isotracker_cookie' % os.getenv('USER'))

post_domain = 'iso.qa.ubuntu.com'
post_url = '/qatracker/admin/addbuild'

ec2_to_testcase_map = {
	'eu-west-1-amd64-ebs': 114,
	'eu-west-1-i386-ebs': 115,
	'us-east-1-amd64-ebs': 118,
	'us-east-1-i386-ebs': 119,
	'us-west-1-amd64-ebs': 116,
	'us-west-1-i386-ebs': 117,
	'eu-west-1-amd64-instance': 92,
	'eu-west-1-i386-instance': 93,
	'us-east-1-amd64-instance': 94,
	'us-east-1-i386-instance': 95,
	'us-west-1-amd64-instance': 85,
	'us-west-1-i386-instance': 86
}

def get_cookie_for_domain(post_domain):
	"""
	Parse a traditional mozilla cookie file and return the first cookie
	matching our domain - assume we only need one, since otherwise we
	need to do a song and dance to pass multiple cookies to httplib
	"""
	handle = open(cookiefile, 'r')
	for line in handle:
		if line.startswith('#'):
			continue
		(domain,junk1,junk2,junk3,junk4,cookiename,cookieval) = \
			line.split()
		if not post_domain.endswith(domain):
			continue
		cookie = '%s=%s' % (cookiename, cookieval)
	handle.close()
	return cookie

def main():
	headers = {
		"Content-type": "application/x-www-form-urlencoded",
	}

	cookie = get_cookie_for_domain(post_domain)
	headers.update({"Cookie": cookie})

	file = sys.argv[1]
	handle = open(file, 'r')

	conn = httplib.HTTPConnection(post_domain)

	for line in handle:
		zone,ami,arch,store = line.split()[0:4]
		if not ami.startswith('ami-'):
			continue
		if store == 'instance-store':
			store = 'instance'
		testcase = ec2_to_testcase_map['%s-%s-%s' % (zone,arch,store)]
		params = urllib.urlencode({
			'isota_case[]': testcase,
			'isota_version': ami,
			'isota_addisoadv': 'Add build(s)',
			# fixme: this needs to be detected
			'isota_release': 111,
		})
		conn.request("POST", post_url, params, headers)
		response = conn.getresponse()
		if response.status != 302:
			print response.status, response.reason
		# flush the response
		response.read()

	conn.close()
	handle.close()

if __name__ == '__main__':
	main()
