#!/usr/bin/python

# Parse removals.txt file
# Copyright (C) 2004, 2005, 2009  Canonical Ltd.
# Authors: James Troup <james.troup@canonical.com>,
#          Colin Watson <cjwatson@ubuntu.com>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

################################################################################

# What kind of genius logs 3.5 years of removal data in
# semi-human-parseable text and nothing else?  Hmm.  That would be me. :(

# MAAAAAAAAAAAAAADNESSSSSSSSSSSSSSSSS

################################################################################

import optparse
import copy
import string
import os
import time
import urllib
import urllib2
import re
import logging
import gzip

import apt_pkg

import lputils
import synclib

CONSUMER_KEY = "process-removals.py"


Debian = None


def parse_removals_file(launchpad, options, removals_file):
    ubuntu_series = launchpad.distributions['ubuntu'].current_series
    ubuntu_archive = ubuntu_series.main_archive
    me = launchpad.me.name

    state = "first separator"
    removal = {}
    for line in removals_file:
        line = line.strip()
        if line == "=========================================================================":
            if state == "done":
                state = "next separator"
            elif state in ("first separator", "next separator"):
                state = "date"
            else:
                raise RuntimeError, ("found separator but state is %s." % state)
        #[Date: Tue,  9 Jan 2001 20:46:15 -0500] [ftpmaster: James Troup]
        elif state in ("date", "next separator"):
            try:
                (date, ftpmaster, unused) = line.split("]")
                state = "removed from"
                removal["ftpmaster"] = ftpmaster.replace("[ftpmaster: ", "")
            except:
                date = line
                state = "broken date"
            date = date.replace("[Date: ", "")
            # Strip the timezone which python's strptime() can't handle [rock on]
            date = ' '.join(date.split()[:-1])
            removal["date"] = time.mktime(time.strptime(date, "%a, %d %b %Y %H:%M:%S"))
        #] [ftpmaster:
        elif state == "broken date":
            (unused1, ftpmaster, unused2) = line.split("]")
            removal["ftpmaster"] = ftpmaster.replace("] [ftpmaster: ", "")
            state = "removed from"
        #Removed the following packages from unstable:
        elif state == "removed from":
            prefix = "Removed the following packages from "
            if not line.startswith(prefix):
                raise RuntimeError, ("state = %s, expected '%s'" % (state, prefix))
            line = line.replace(prefix, "")[:-1]
            line = line.replace(" and", "")
            line = line.replace(",", "")
            removal["suites"] = line.split()
            state = "before packages"
        elif state == "before packages" or state == "after packages": 
            if line:
                raise RuntimeError, ("state = %s, expected ''" % (state))
            if state == "before packages":
                state = "packages"
            elif state == "after packages":
                state = "reason prefix"
        #xroach |      4.0-8 | source, alpha, arm, hppa, i386, ia64, m68k, mips, mipsel, powerpc, s390, sparc
        #Closed bugs: 158188
        elif state == "packages":
            if line.find("|") != -1:
                (package, version, architectures) = map(string.strip, line.split("|"))
                architectures = map(string.strip, architectures.split(","))
                removal.setdefault("packages", [])
                removal["packages"].append([package, version, architectures])
            elif line.startswith("Closed bugs: "):
                line = line.replace("Closed bugs: ", "")
                removal["closed bugs"] = line.split()
                state = "after packages"
            elif not line:
                state = "reason prefix"
            else:
                raise RuntimeError, ("state = %s, expected package list or 'Closed bugs:'" % (state))
        #------------------- Reason -------------------
        elif state == "reason prefix":
            expected = "------------------- Reason -------------------"
            if not line == expected:
                raise RuntimeError, ("state = %s, expected '%s'" % (state, expected))
            state = "reason"
        #RoSRM; license problems.
        #----------------------------------------------
        elif state == "reason":
            if line == "----------------------------------------------":
                state = "done"
                do_removal(ubuntu_series, ubuntu_archive, me, options, removal)
                removal = {}
            else:
                removal.setdefault("reason", "")
                removal["reason"] += "%s\n" % line

        # nothing should go here


def do_removal(ubuntu_series, ubuntu_archive, me, options, removal):
    if options.removed_from and options.removed_from not in removal["suites"]:
        return
    if options.date and int(options.date) > int(removal["date"]):
        return
    if options.architecture:
        for architecture in re.split(r'[, ]+', options.architecture):
            package_list = copy.copy(removal["packages"])
            for entry in package_list:
                (package, version, architectures) = entry
                if architecture not in architectures:
                    removal["packages"].remove(entry)
        if not removal["packages"]:
            return

    for entry in removal["packages"]:
        (package, version, architectures) = entry
        if Debian.has_key(package):
            logging.info("%s (%s): back in sid - skipping.", package, version)
            continue

        spp = []
        spp.extend(ubuntu_archive.getPublishedSources(
            distro_series=ubuntu_series, pocket='Release',
            source_name=package, exact_match=True, status='Published'))

        if not len(spp):
            logging.debug("%s (%s): not found", package, version)
            continue

        logging.info("%s (%s/%s)", package, version, spp[-1].source_package_version)

        args = ["-u", me]
        if options.no_action:
            args.append("-n")
        removal["reason"] = removal["reason"].rstrip('\n')
        try:
            bugs = ';' + ','.join([" Debian bug #%s" % item for item in removal["closed bugs"]])
        except KeyError:
            bugs = ''		
        args.extend(["-m", "(From Debian) %s" % removal["reason"].rstrip('\n') + bugs])
        args.append(package)

        synclib.request('lp-remove-package.py', args=args,
                        pipe_stdout=False).communicate()


def fetch_removals_file():
    logging.debug("Fetching removals.txt")
    return urllib2.urlopen("http://ftp-master.debian.org/removals.txt")


def read_sources():
    global Debian
    Debian = {}

    logging.debug("Reading Debian sources")
    for component in "main", "contrib", "non-free":
        filename = "Debian_unstable_%s_Sources" % component
        if not os.path.exists(filename):
            url = "http://ftp.debian.org/debian/dists/unstable/%s/source/Sources.gz" % component
            logging.info("Fetching %s" % url)
            fetched, headers = urllib.urlretrieve(url)
            out = open(filename, 'w')
            gzip_handle = gzip.GzipFile(fetched)
            while True:
                block = gzip_handle.read(65536)
                if not block:
                    break
                out.write(block)
            out.close()
            gzip_handle.close()
        sources_filehandle = open(filename)
        Sources = apt_pkg.ParseTagFile(sources_filehandle)
        while Sources.Step():
            pkg = Sources.Section.Find("Package")
            version = Sources.Section.Find("Version")

            if Debian.has_key(pkg) and apt_pkg.VersionCompare(Debian[pkg]["version"], version) > 0:
                continue

            Debian[pkg] = {}
            Debian[pkg]["version"] = version
        sources_filehandle.close()


def parse_options():
    parser = optparse.OptionParser()

    parser.add_option("-l", "--launchpad", dest="launchpad_instance",
                      default="edge")

    parser.add_option("-a", "--architecture", metavar="ARCH",
                      help="remove from ARCH")
    parser.add_option("-d", "--date", metavar="DATE",
                      help="only those removed since DATE")

    parser.add_option("-r", "--removed-from", metavar="SUITE",
                      help="only those removed from SUITE (aka distrorelease)")

    parser.add_option("-f", "--filename",
                      help="parse FILENAME")

    parser.add_option("-n", "--no-action", action="store_true",
                      help="don't do anything")
    parser.add_option("-v", "--verbose", action="store_true",
                      help="emit verbose debugging messages")

    parser.add_option("--print-commands", action="store_true",
                      help="print lp-remove-package.py commands")

    options, args = parser.parse_args()
    if len(args):
        parser.error("no arguments expected")

    if options.date:
        try:
            int(options.date)
        except ValueError:
            options.date = time.mktime(time.strptime(options.date, "%Y-%m-%d"))
    else:
        options.date = time.mktime(time.strptime("2009-02-01", "%Y-%m-%d"))
    if not options.removed_from:
        options.removed_from = "unstable"
    if not options.architecture:
        options.architecture = "source"

    if options.print_commands:
        os.environ['SYNCLIB_DEBUG'] = '1'
    if options.verbose:
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig(level=logging.INFO)

    return options, args


def main():
    apt_pkg.init()

    """Initialization, including parsing of options."""
    options, args = parse_options()

    launchpad = lputils.lpfactory(options.launchpad_instance, CONSUMER_KEY)

    read_sources()

    if options.filename:
        removals_file = open(options.filename)
    else:
        removals_file = fetch_removals_file()
    parse_removals_file(launchpad, options, removals_file)
    removals_file.close()


if __name__ == '__main__':
    main()
