#!/usr/bin/python

# Copyright 2008, 2009 Canonical Ltd.

"""Adjust SRU bugs after accepting the corresponding update."""

import sys
import os
from optparse import OptionParser
import re

sys.path.insert(0, os.path.join(
    os.path.dirname(__file__), 'lib'))

import launchpadlib.errors

import lputils


CONSUMER_KEY = "sru-accept.py"

bug_target_re = re.compile(
    r'/ubuntu/(?:(?P<suite>[^/]+)/)?\+source/(?P<source>[^/]+)$')


def get_main_archive(launchpad):
    distros = launchpad.distributions
    ubuntu = distros['ubuntu']
    return ubuntu.main_archive


def process_bug(launchpad, options, num):
    bug = launchpad.bugs[num]

    for task in bug.bug_tasks:
        # Ugly; we have to do URL-parsing to figure this out.
        # /ubuntu/+source/foo can be fed to launchpad.load() to get a
        # distribution_source_package, but /ubuntu/hardy/+source/foo can't.
        match = bug_target_re.search(task.target.self_link)
        if (not match or
            (options.package and match.group('source') != options.package)):
            print "Ignoring task %s in bug %s" % (task.target, num)
            continue
        if (match.group('suite') == options.target and
            task.status not in ("Invalid", "Won't Fix", "Fix Committed", "Fix Released")):
            task.status = "Fix Committed"
            task.lp_save()
            print "Success: task %s in bug %s" % (task.target, num)

    bug.subscribe(person=launchpad.people['ubuntu-sru'])
    bug.subscribe(person=launchpad.people['sru-verification'])

    for t in ('verification-failed', 'verification-done'):
        if t in bug.tags:
            # this dance is needed due to
            # https://bugs.launchpad.net/launchpadlib/+bug/254901
            tags = bug.tags
            tags.remove(t)
            bug.tags = tags
    bug.lp_save()

    if 'verification-needed' not in bug.tags:
        # this dance is needed due to
        # https://bugs.launchpad.net/launchpadlib/+bug/254901
        tags = bug.tags
        tags.append('verification-needed')
        bug.tags = tags
        bug.lp_save()

    if options.package:
        text = 'Accepted %s into ' % options.package
    else:
        text = 'Accepted into '
    text += ('%s-proposed, the package will build now and be available in a few hours. '
             'Please test and give feedback here. See '
             'https://wiki.ubuntu.com/Testing/EnableProposed for '
             'documentation how to enable and use -proposed. Thank you in '
             'advance!' % options.target)
    bug.newMessage(content=text, subject='Please test proposed package')


if __name__ == '__main__':
    parser = OptionParser(usage="Usage: %prog [options] bug [bug ...]")

    parser.add_option("-l", "--launchpad", dest="launchpad_instance",
                      default="edge")
    parser.add_option('-s', dest='target',
                      help='accept for SUITE(-proposed) instead of lucid',
                      default='lucid',
                      metavar='SUITE')
    parser.add_option('-p', dest='package',
                      help='only change tasks for a particular source package',
                      default=None,
                      metavar='SRCPACKAGE')

    options, args = parser.parse_args()
    if options.target.endswith('-proposed'):
        options.target = options.target[:-9]

    launchpad = lputils.lpfactory(options.launchpad_instance, CONSUMER_KEY)
    try:
        for num in args:
            process_bug(launchpad, options, num)
    except launchpadlib.errors.HTTPError, err:
        print "There was an error:"
        print err.content
