# Copyright 2009 Canonical Ltd.

# Some day, we'll have a proper API for doing sync/backport operations in
# Launchpad. In the meantime, we have to shove files through a privileged
# back door (specifically, one that doesn't check signatures, so that we
# don't have to re-GPG-sign all this stuff) to make it work properly. This
# is a very simple client-side interface to this back door. It only works if
# you have lp_archive access on the publishing machine.

import os
import subprocess

if 'UBUNTU_ARCHIVE_HOST' in os.environ:
    remote_host = os.environ['UBUNTU_ARCHIVE_HOST']
else:
    remote_host = 'cocoplum.canonical.com'


# Aggravatingly, ssh runs the command you give it through the shell, so we
# have to shell-escape everything.
def shell_escape(arg):
    if arg:
        return "'%s'" % arg.replace("'", "'\\''")
    return 'None'


def request(op_type, args=None, pipe_stdout=True):
    backend = op_type
    if op_type == 'backport':
        directory = '/home/lp_archive/backports'
        backend = 'backport-source-backend'
    elif op_type == 'sync':
        directory = '/home/lp_archive/syncs'
        backend = 'sync-source.py'
    elif op_type == 'flush-backports':
        directory = '/home/lp_archive/backports'
    elif op_type == 'flush-syncs':
        directory = '/home/lp_archive/syncs'
    else:
        directory = '.'

    command = ['ssh', '-q', '-t', remote_host,
               '/home/lp_archive/bin/sync-backend', directory, backend]
    if args is not None:
        command.append(' '.join([shell_escape(arg) for arg in args]))
    if 'SYNCLIB_DEBUG' in os.environ:
        print ' '.join(command[:-1]), shell_escape(command[-1])
        command = ['true']
    if pipe_stdout:
        return subprocess.Popen(command, stdout=subprocess.PIPE,
                                stderr=subprocess.STDOUT)
    else:
        return subprocess.Popen(command)


def do_sync(op_type, source, extra_opts):
    output = []
    sync_done = True
    opts = list(extra_opts)
    opts.append(source)
    handle = request(op_type, opts)
    for line in handle.stdout:
        print line,
        if (not line.startswith('Getting binaries for') and
            not line.startswith('No handlers could be found for logger')):
            output.append(line)
        if (line.startswith('[NOT Updating - Modified]') or
            line.startswith('[BLACKLISTED]') or
            line.startswith('[Nothing to update')):
            sync_done = False
    return (handle.wait() == 0 and sync_done, output)
